# papmolmod.py
# python script to draw linouts for molecular models with paper
# requires ROOT libraries from root.cern.ch
# licence Attribution-NonCommercial-ShareAlike
# Rolf Oldeman 18/02/2020

from ROOT import TCanvas,TLine,TPolyLine,TText
from ROOT import gStyle,gPad
from math import *
from array import array

degree=pi/180.0

pagewidth=20.0
pageheight=26.0
hscale=1.00
vscale=1.05

grid=False
label=True

lipwidth=0.5
lipangle=30*degree

gStyle.SetLineScalePS(1.5)

line=TLine()
line.SetLineWidth(1)

plines=[]

text=TText()
text.SetTextFont(132);
text.SetTextSize(0.03);


def move(d):
    global x,y
    x+=d*cos(theta)
    y+=d*sin(theta)

def turn(angle):
    global theta
    theta+=angle

def drawline(l):
    line.DrawLine(x,y,x+l*cos(theta),y+l*sin(theta))

# draw regular polygon turning to the left (n>0) or right (n<0)
def drawregpoly(size,n,lips="",fill=False):
    lips+=(" "*abs(n))
    xp=[]; yp=[]
    xp.append(x); yp.append(y)
    for i in range(abs(n)):
        if lips[i]=='L':drawisotrap(size,lipwidth,lipangle)
        if lips[i]=='R':drawisotrap(size,-lipwidth,-lipangle)
        move(size)
        xp.append(x); yp.append(y)
        turn((360.0/n)*degree)
    plines.append(TPolyLine(abs(n)+1,array('f',xp),array('f',yp)))
    plines[-1].SetLineStyle(1)
    plines[-1].SetLineColor(1)
    plines[-1].SetFillColor(0)
    plines[-1].SetFillStyle(1001)
    plines[-1].Draw('F')
    plines[-1].Draw('')
    if fill:
        plines[-1].SetFillColor(fillcolor)
        plines[-1].SetFillStyle(fillstyle)
        plines[-1].Draw('F')
        plines[-1].Draw('')

# draw polygon with alternating lengths turning to the left (n>0) or right (n<0)
def drawaltpoly(s1,s2,n,lips="",fill=False):
    lips+=(" "*abs(n))
    xp=[]; yp=[]
    xp.append(x); yp.append(y)
    for i in range(abs(n)):
        if i%2==0:size=s1
        if i%2==1:size=s2
        if lips[i]=='L':drawisotrap(size,lipwidth,lipangle)
        if lips[i]=='R':drawisotrap(size,-lipwidth,-lipangle)
        move(size)
        xp.append(x); yp.append(y)
        turn((360/n)*degree)
    plines.append(TPolyLine(abs(n)+1,array('f',xp),array('f',yp)))
    plines[-1].SetLineStyle(1)
    plines[-1].SetLineColor(1)
    plines[-1].SetFillColor(0)
    plines[-1].SetFillStyle(1001)
    plines[-1].Draw('F')
    plines[-1].Draw('')
    if fill:
        plines[-1].SetFillColor(fillcolor)
        plines[-1].SetFillStyle(fillstyle)
        plines[-1].Draw('F')
        plines[-1].Draw('')

# draw isosceles trapezoid to the left (h,a positive) or right (h,a negative)
def drawisotrap(length,height,angle,lips="",fill=False):
    if height>0.0: height=min(height,length/2*tan(angle))
    #if height<0.0: height=-min(-height,length/2*tan(angle))
    lips+="    "
    xp=[]; yp=[]
    xp.append(x); yp.append(y)
    l=length
    if lips[0]=='L':drawisotrap(l,lipwidth,lipangle)
    if lips[0]=='R':drawisotrap(l,-lipwidth,-lipangle)
    if lips[0]==' ':drawline(l)
    move(l)
    xp.append(x); yp.append(y)
    turn(180*degree-angle)
    l=height/sin(angle)
    if lips[1]=='L':drawisotrap(l,lipwidth,lipangle)
    if lips[1]=='R':drawisotrap(l,-lipwidth,-lipangle)
    if lips[1]==' ':drawline(l)
    move(l)
    xp.append(x); yp.append(y)
    turn(angle)
    l=length-2*height/tan(angle)
    if lips[2]=='L':drawisotrap(l,lipwidth,lipangle)
    if lips[2]=='R':drawisotrap(l,-lipwidth,-lipangle)
    if lips[2]==' ':drawline(l)
    move(l)
    xp.append(x); yp.append(y)
    turn(angle)
    l=height/sin(angle)
    if lips[3]=='L':drawisotrap(l,lipwidth,lipangle)
    if lips[3]=='R':drawisotrap(l,-lipwidth,-lipangle)
    if lips[3]==' ':drawline(l)
    move(l)
    xp.append(x); yp.append(y)
    turn(180*degree-angle)
    plines.append(TPolyLine(5,array('f',xp),array('f',yp)))
    #plines[-1].SetLineStyle(1)
    #plines[-1].SetLineColor(1)
    #plines[-1].SetFillColor(0)
    #plines[-1].SetFillStyle(1001)
    #plines[-1].Draw('F')
    #plines[-1].Draw('')
    if fill:
        plines[-1].SetFillColor(fillcolor)
        plines[-1].SetFillStyle(fillstyle)
        plines[-1].Draw('F')
        #plines[-1].Draw('')

# draw a regular tube
# ls is a tuple with the nstep+1 lenghts of the sides
# ds is a tuple with the nstep distances of the steps
# the array fills determines which polygons are filled and which not
def regtube(nside,ls,ds,fills=[]):
    global x,y,theta
    nstep=len(ls)-1
    fills+=[False]*(2+nside*nstep)
    ifill=0
    drawregpoly(ls[0],nside,"",fills[ifill])
    ifill+=1
    for iside in range(nside): 
        status=x,y,theta
        for istep in range(nstep):
            alpha=acos((ls[istep]-ls[istep+1])/(2*ds[istep]))
            if iside==nside/2: lips=" L L"
            if iside<nside/2:  lips="   L"
            if iside>nside/2:  lips=" L  "
            if iside==0:       lips="    "
            if istep==nstep-1 and iside != 0: lips=lips[0:2]+"L"+lips[3]
            drawisotrap(ls[istep],-ds[istep]*sin(alpha),-alpha,lips,fills[ifill])
            ifill+=1
            turn(-alpha)
            move(ds[istep])
            turn(alpha)
            if istep==nstep-1 and iside==0:
                drawregpoly(ls[nstep],-nside,"",fills[ifill])
                ifill+=1
                
        x,y,theta=status
        move(ls[0])
        turn((360.0/nside)*degree)
           

# draw an alternated tube
# ls is a tuple with the 2*nstep+2 lenghts of the even/odd sides
# negative values indicate that no line is to be drawn
# ds is a tuple with the nstep distances of the steps
def alttube(nside,ls,ds,fills=[]):
    global x,y,theta
    nstep=len(ls)/2-1
    fills+=[False]*(2+nside*nstep)
    ifill=0
    drawaltpoly(ls[0],ls[1],nside,"",fills[ifill])
    ifill+=1
    for iside in range(nside): 
        status=x,y,theta
        for istep in range(nstep):
            l1=ls[2*istep+iside%2]
            l2=ls[2*istep+iside%2+2]
            alpha=acos((abs(l1)-abs(l2))/(2*ds[istep]))
            if iside==nside/2: lips=" L L"
            if iside<nside/2:  lips="   L"
            if iside>nside/2:  lips=" L  "
            if iside==0:       lips="    "
            if istep==nstep-1 and iside != 0: lips=lips[0:2]+"L"+lips[3]
            if l1<0: lips="X"+lips[1:]
            if l2<0: lips=lips[:2]+"X"+lips[3]
            drawisotrap(abs(l1),-ds[istep]*sin(alpha),-alpha,lips,fills[ifill])
            ifill+=1
            turn(-alpha)
            move(ds[istep])
            turn(alpha)
            if istep==nstep-1 and iside==0:
                drawaltpoly(ls[2*nstep],ls[2*nstep+1],-nside,"",fills[ifill])
                ifill+=1
                
        x,y,theta=status
        move(ls[0+iside%2])
        turn((360.0/nside)*degree)
           

    
def drawgrid():
    line.SetLineStyle(3)
    for y in range(27): line.DrawLine(0.0,y,pagewidth,y)
    for x in range(21): line.DrawLine(x,0.0,x,pageheight)
    line.SetLineStyle(1)

def drawlabel():
    text.SetTextSize(0.02);
    text.DrawText(1.0,0.5,"https://www.instructables.com/id/Paper-Molecular-Models")

    
#gStyle.SetPaperSize(pagewidth,pageheight)
c1=TCanvas("c1","c1",int(pagewidth*hscale*20),int(pageheight*vscale*20))
gPad.Range(0.0,0.0,pagewidth,pageheight)


# hydrogen
c1.Clear()

text.SetTextSize(0.03);
text.DrawText(1.0,25.0,"hydrogen")

fillcolor=0
fillstyle=1001

if grid: drawgrid()
if label: drawlabel()

x,y,theta=10.0,6.0,90*degree
regtube(6,(3.0,2.0),(3.0,))

x,y,theta=10.0,21.0,-90*degree
regtube(6,(3.0,2.0),(3.0,))

c1.Print("hydrogen.pdf")


# linear
c1.Clear()

fillcolor=0
fillstyle=1001

text.SetTextSize(0.03);
text.DrawText(1.0,25.0,"linear")

if grid: drawgrid()
if label: drawlabel()

x,y,theta=9.0,13.0,0*degree
regtube(6,(3.0,4.0,4.0,3.0),(2.0,2.0,2.0))

c1.Print("linear.pdf")


# trigonal
c1.Clear()
fillcolor=0
fillstyle=1001

text.SetTextSize(0.03);
text.DrawText(1.0,25.0,"trigonal")

if grid: drawgrid()
if label: drawlabel()

x,y,theta=8.5,13.0,0*degree
l1,l2=3.0,1.5
alttube(6,(l1,l2+l1/2, -2*l1,l2, l1,l2+l1/2),(l1,l1))

c1.Print("trigonal.pdf")


# tetrahedral
c1.Clear()
fillcolor=0
fillstyle=1001

text.SetTextSize(0.03);
text.DrawText(1.0,25.0,"tetrahedral")

if grid: drawgrid()
if label: drawlabel()

x,y,theta=8.5,12.5,0*degree
l1,l2=3.0,1.5
alttube(6,(l1,l1, l1,-(l1+l2), -2*l1,l2, l1,l2),(l2,l1,l1))


c1.Print("tetrahedral.pdf)")



